Class VFXListHelper.AbstractHelper<T, C extends VFXCell<T>>
java.lang.Object
io.github.palexdev.virtualizedfx.base.VFXContainerHelper.VFXContainerHelperBase<T, VFXList<T,C>>
io.github.palexdev.virtualizedfx.list.VFXListHelper.AbstractHelper<T,C>
- All Implemented Interfaces:
VFXContainerHelper<T, VFXList<T,C>>, VFXListHelper<T, C>
- Direct Known Subclasses:
VFXListHelper.HorizontalHelper, VFXListHelper.VerticalHelper
- Enclosing interface:
VFXListHelper<T, C extends VFXCell<T>>
public abstract static class VFXListHelper.AbstractHelper<T, C extends VFXCell<T>>
extends VFXContainerHelper.VFXContainerHelperBase<T, VFXList<T,C>>
implements VFXListHelper<T,C>
Extension of
VFXContainerHelper.VFXContainerHelperBase which also implements VFXListHelper.
Defines common properties and operations for the two concrete implementations VFXListHelper.VerticalHelper and
VFXListHelper.HorizontalHelper, such as:
- the range of items to display as a IntegerRangeProperty
- the total number of cells in the viewport
-
Nested Class Summary
Nested classes/interfaces inherited from interface VFXContainerHelper
VFXContainerHelper.VFXContainerHelperBase<T,C> Nested classes/interfaces inherited from interface VFXListHelper
VFXListHelper.AbstractHelper<T,C>, VFXListHelper.HorizontalHelper<T, C>, VFXListHelper.VerticalHelper<T, C> -
Property Summary
PropertiesTypePropertyDescriptionio.github.palexdev.mfxcore.base.properties.range.IntegerRangePropertySpecifies the range of items present in the list.Properties inherited from class VFXContainerHelper.VFXContainerHelperBase
maxHScroll, maxVScroll, viewportPosition, virtualMaxX, virtualMaxYProperties inherited from interface VFXContainerHelper
maxHScroll, maxVScroll, viewportPosition, virtualMaxX, virtualMaxY -
Field Summary
FieldsModifier and TypeFieldDescriptionprotected final io.github.palexdev.mfxcore.base.properties.range.IntegerRangePropertyFields inherited from class VFXContainerHelper.VFXContainerHelperBase
container, maxHScroll, maxVScroll, viewportPosition, virtualMaxX, virtualMaxY, vmxBinding, vmyBinding -
Constructor Summary
Constructors -
Method Summary
Modifier and TypeMethodDescriptionio.github.palexdev.mfxcore.base.properties.range.IntegerRangePropertySpecifies the range of items present in the list.inttotalNum()Methods inherited from class VFXContainerHelper.VFXContainerHelperBase
createBindings, createMaxHScrollBinding, createMaxVScrollBinding, createVirtualMaxXBinding, createVirtualMaxYBinding, dispose, getContainer, invalidateVirtualSizes, maxHScrollProperty, maxVScrollProperty, viewportPositionProperty, virtualMaxXProperty, virtualMaxYPropertyMethods inherited from class Object
clone, equals, finalize, getClass, hashCode, notify, notifyAll, toString, wait, wait, waitMethods inherited from interface VFXContainerHelper
dispose, getContainer, getMaxHScroll, getMaxVScroll, getViewportPosition, getVirtualMaxX, getVirtualMaxY, indexToItem, invalidatePos, invalidateVirtualSizes, maxHScrollProperty, maxVScrollProperty, viewportPositionProperty, virtualMaxXProperty, virtualMaxYPropertyMethods inherited from interface VFXListHelper
computeSize, firstVisible, getTotalCellSize, indexToCell, itemToCell, lastVisible, layout, range, scrollBy, scrollToIndex, scrollToPixel, visibleNum
-
Property Details
-
range
public io.github.palexdev.mfxcore.base.properties.range.IntegerRangeProperty rangeProperty- Specified by:
rangePropertyin interfaceVFXListHelper<T, C extends VFXCell<T>>- Returns:
- the
rangeproperty - See Also:
-
-
Field Details
-
range
protected final io.github.palexdev.mfxcore.base.properties.range.IntegerRangeProperty range
-
-
Constructor Details
-
AbstractHelper
-
-
Method Details
-
totalNum
-
rangeProperty
public io.github.palexdev.mfxcore.base.properties.range.IntegerRangeProperty rangeProperty()Description copied from interface:VFXListHelperSpecifies the range of items present in the list. This also takes into account buffer items, seeVFXListHelper.visibleNum()andVFXListHelper.totalNum()- Specified by:
rangePropertyin interfaceVFXListHelper<T, C extends VFXCell<T>>- Returns:
- the
rangeproperty
-